

#ifndef __Vector__
#define __Vector__

#include "Object.h"
#include "Enumeration.h"

/**
 * The Vector class implements a growable array of objects. Like an array, it contains components 
 * that can be accessed using an integer index. However, the size of a Vector can grow or shrink as 
 * needed to accommodateadding and removing items after the Vector has been created. 
 *
 * Each vector tries to optimize storage management by maintaining a capacity and a capacityIncrement.
 * The capacity is always at least as large as the vector size; it is usually larger because as 
 * components are added to the vector, the vector's storage increases in chunks the size of 
 * capacityIncrement. An application can increase the capacity of a vector before inserting a large 
 * number of components; this reduces the amount of incremental reallocation. 
 */

class Vector : public Object, public Enumeration
{
protected:

  Object **vec;
  int elementCount;
  int elementData;
  int incrementSize;
  int readElements;
  void trim(); /*shrink's the vector */ 
  void resize(); /*resize the vector on a given incrementSize*/

public:
  Vector(); /** Generic Constructor */
  Vector(int); /**Constructor with a given amount of Capacity */
  Vector(int, int); /** Constructor with a given amount of Capacity and incrementSize */ 
  Vector(Vector &); /**CopyConstructor */
  virtual ~Vector(); /** Generic Destructor */
  void addElement(Object *); /** Adds the specified component to the end of this vector */
  void setElementAt(Object *, int); /**Overwrites a given element at Position x **/
  Object *elementAt(int); /** Returns the component at the specified index. */ 
  Enumeration *elements(); /** Returns an enumeration of the components of this vector. */
  bool hasMoreElements(); /** Tests if this enumeration contains more elements.*/
  Object *nextElement(); /** Returns the next element of this enumeration. */
  void insertElementAt(Object *, int);/** Inserts the specified object as a component in this vector  at the specified index. */
  bool isEmpty(); /** Tests if this vector has no components. */
  bool removeElementAt(int); /** Deletes the component at the specified index. */ 
  bool removeElement(Object *); /**Deletes the last occurence of the given Object */
  virtual String toString(); /** Returns a string representation of this vector. */
  int size();
  void setIncrementSize(int);
  int getIncrementSize();
  int getElementCount();
  void ensureCapacity(int);
  virtual bool equals(Vector *);
};

ostream &operator<<(ostream &,Vector &);

#endif
